const pluginInit = require('./pluginInit');

/**
 * 在 plugins/index 中进行注入，如果不存，则创建并注入
 * @param {api} api
 * @param {string} importFrom: import xxx from '\<importFrom\>';
 * @param {string} defaultImport: import \<defaultImport\> from '\<importFrom\>';
 * @param {string[]} importList:
 * import \<defaultImport\>, { ...\<importList\> } from '\<importFrom\>';
 */
module.exports = (api, importFrom, defaultImport, importList) => {
  pluginInit(api);

  let pluginEntry;

  if (api.hasPlugin('typescript')) {
    pluginEntry = 'src/plugins/index.ts';
  } else {
    pluginEntry = 'src/plugins/index.js';
  }

  api.render((files) => {
    const fileList = files;

    if (!fileList[pluginEntry]) {
      fileList[pluginEntry] = '';
    }
  });

  api.transformScript(pluginEntry, require('./useInjectImport'), {
    from: importFrom,
    default: defaultImport,
    importList,
  });
};
