const { accessSync, writeFileSync, readFileSync } = require('fs');
const path = require('path');
const dotenv = require('dotenv');

function genImport(j, from, defaultImport, ...namedImport) {
  const importList = [];

  if (defaultImport) {
    importList.push(j.importDefaultSpecifier(j.identifier(defaultImport)));
  }

  importList.push(
    ...namedImport.map((key) => j.importSpecifier(j.identifier(key))),
  );
  const newImport = j.importDeclaration(importList, j.stringLiteral(from));

  return newImport;
}

function writeFile(filePath, content) {
  let ret = true;

  try {
    writeFileSync(filePath, content, { encoding: 'utf8' });
  } catch (e) {
    ret = false;
  }

  return ret;
}

function readFile(filePath) {
  let ret = '';

  try {
    ret = readFileSync(filePath, { encoding: 'utf8' });
  } catch (e) {
    ret = '';
  }

  return ret;
}

function readEnv(filePath) {
  let ret = {};

  try {
    ret = dotenv.parse(Buffer.from(readFile(filePath)));
  } catch (e) {
    console.warn('readEnv error:', e.message);
    ret = {};
  }

  return ret;
}

function readEnvFromContent(content) {
  let ret = {};

  try {
    ret = dotenv.parse(content);
  } catch (e) {
    console.warn('readEnv error:', e.message);
    ret = {};
  }

  return ret;
}

function genNamedExport(j, variableName) {
  return j.exportSpecifier.from({
    exported: j.identifier(variableName),
    exportKind: 'value',
    local: j.identifier(variableName),
  });
}

function resolve(base) {
  return path.resolve(process.cwd(), base);
}

function isTargetExist(filePath) {
  const fullPath = resolve(filePath);

  try {
    accessSync(fullPath);

    return true;
  } catch (e) {
    return false;
  }
}

function checkPluginExist() {
  return isTargetExist('./src/plugins');
}

function checkQiankunExist() {
  return isTargetExist('./src/qiankun');
}

function buildEnvContent(values) {
  let content = '';

  Object.keys(values).forEach((key) => {
    content += `${key}=${values[key]}\n`;
  });

  return content;
}

module.exports = {
  genImport,
  writeFile,
  readFile,
  readEnv,
  readEnvFromContent,
  genNamedExport,
  resolve,
  isTargetExist,
  checkPluginExist,
  checkQiankunExist,
  buildEnvContent,
};
